// ----------------------------------------------------------------------------------
/* [z-11 project]
 * (c) 1999 Copyright Malekith
 * <malekith@topnet.pl> <http://malekith.topnet.pl>
 * For info about copying see file copying.z11.
 *
 * file "TextScr.cc", created on 18/06/99.
 * descr : console - gfx stuff for terminal
 */
// ----------------------------------------------------------------------------------

#include "z11.h"
#include "TextScr.h"

FILEINFO(TextScr);

// ----------------------------------------------------------------------------------
TextScreenEmu::TextScreenEmu() : ready(false), lineBuf(0), buffer(0) { }
// ----------------------------------------------------------------------------------
void TextScreenEmu::init() {
  try {
    bgd.load(gfxData+"misc/console.pcx");
  } catch(EFile & e) {
    bgd.setSize(100,100);
    bgd.clear();
  }
  bgd.resize(screen.w,screen.h);
  lineBuf=new char[screen.w+1]; // fonts can be even 1 pixel wide...
  loadFont("rom8x8");
  setTextColor(0xffffff,0.7);
};
// ----------------------------------------------------------------------------------
TextScreenEmu::~TextScreenEmu() {
  delete [] buffer;
  delete [] lineBuf;
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::setTextColor(int color, float v) {
  textColor=color; colorDiff=v;
}
// ----------------------------------------------------------------------------------
bool TextScreenEmu::loadFont(XStr name) {
  font.load(gfxData+"fonts/"+name+".fnt");
  int nw=screen.w/font.w-4, nh=(int)((0.9*screen.h)/font.h);
  char * nBuf = new char[nw*nh+1];
  memset(nBuf,' ',nw*nh);
  if(buffer) {
    int s=h, d=nh;
    while(--s>=0 && --d>=0)
      for(int j=0; j<(nw<?w); j++)
        nBuf[d*nw+j]=buffer[s*w+j];
  }
  w=nw, h=nh;
  if(buffer) delete [] buffer;
  buffer=nBuf;
  return true;
}
// ----------------------------------------------------------------------------------
bool showFast=false;
void TextScreenEmu::slide(bool down) {
  const int loginTime=ups/3;
  timer_t stop=timer()+loginTime,t;
  int x=ups/20;
  if(down) dy=-bgd.h;
  else dy=0;
  while((t=timer())<stop) {
    if(down) dy+=bgd.h*x/loginTime;
    else     dy-=bgd.h*x/loginTime;
    if(dy>0) dy=0;
    screen.vsync();
    if(!down && -dy) {
      Bmp t(backup.w,-dy);
      t.get(backup,0,bgd.h+dy);
      t.put(0,bgd.h+dy);
      // blit(backup,screen,0,bgd.buffer->h+dy,0,bgd.buffer->h+dy,backup->w,-dy);
    } 
    drawAll();
    x=timer()-t;
  }
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::login() {
  if(ready) return;
  backup.setSize(bgd.w,bgd.h);
  backup.get(0,0);
  ready=true;
  if(!showFast) slide(true);
  dy=0;
  drawAll();
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::logout() {
  if(!showFast) slide(false);
  backup.put(0,0);
  backup.free();
  ready=false;
}
// ----------------------------------------------------------------------------------
char TextScreenEmu::convert(char in) {
  switch(in) {
    case 0 ... 31: return ' ';
    default:       return in;
  }
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::dumpLine(int y) {
  char * ptr = buffer+y*w;
  char * out = lineBuf;
  for(int i=0; i<w; i++) *out++=convert(*ptr++);
  *out=0;
}
// ----------------------------------------------------------------------------------
const int margin=2;
void TextScreenEmu::drawAll() {
  if(!ready) return;
  Bmp tmp = bgd;
  for(int i=0; i<h; i++) {
    dumpLine(i);
    Bmp(lineBuf,font,textColor,
         Font::normal,colorDiff).putS(tmp,margin*font.w,i*font.h);
  }
  tmp.put(0,dy);
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::drawLine(int y) {
  if(!ready) return;
  Bmp tmp(bgd.w,font.h);
  tmp.get(bgd,0,font.h*y);
  dumpLine(y);
  Bmp(lineBuf,font,textColor,
       Font::normal,colorDiff).putS(tmp,margin*font.w,0);
  tmp.put(0,font.h*y);
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::scroll() {
  memmove(buffer,buffer+w,w*(h-1));
  memset(buffer+w*(h-1),' ',w);
  drawAll();
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::add(XStr line) {
  memmove(buffer,buffer+w,w*(h-1));
  memset(buffer+w*(h-1),' ',w);
  strncpy(buffer+w*(h-1),line.chars(),w);
  drawAll();
}
// ----------------------------------------------------------------------------------
void TextScreenEmu::setLast(XStr line) {
  memset(buffer+w*(h-1),' ',w);
  strncpy(buffer+w*(h-1),line.chars(),w);
  drawLine(h-1);
}
// ----------------------------------------------------------------------------------



