// ----------------------------------------------------------------------------------
/* [z-11 project]
 * (c) 1999 Copyright Malekith
 * <malekith@topnet.pl> <http://malekith.topnet.pl>
 * For info about copying see file copying.z11.
 *
 * file "Block.h", created on 20/12/98.
 * descr : my dynamic array class definition
 * everything is inline to avoid problems with unresolved externals
 * know, that's not the best idea, but ...
 */
// ----------------------------------------------------------------------------------

#ifndef BlockH
#define BlockH

// ----------------------------------------------------------------------------------
template <class T>
class Block {
    T *v;
    int mem,sizeD;
  public:
    Block(Block & b) { sizeD=mem=b.size(); v=new T[mem+1];  for(int i=0; i<size(); i++) v[i]=b[i]; }
    Block & operator =(Block & b) { delete [] v; sizeD=mem=b.size(); v=new T[mem+1]; for(int i=0; i<size(); i++) v[i]=b[i]; return *this; }
    Block() : mem(10), sizeD(0) { v=new T[mem+1]; }
    int size() { return sizeD; }
    void reserve(int n) {
      if(n>mem) { int newmem=mem+1; while(n>newmem) newmem*=2;
        T * np = new T[newmem+1]; for(int i=0; i<n && i<size(); i++) np[i]=v[i];
        delete [] v; v=np; mem=newmem;
      }
      if(n>sizeD) sizeD=n;
    }
    int index(const T& k) { for(int i=0; i<size(); i++) if(k==v[i]) return i; return -1; }
    T & operator[](int i) {
#     ifdef BlockCheck
        if(i<0 || i>=size()) throw EBlock(xprintf("Index violation (i==%d, size==%d)",i,size()));
#     endif
      return v[i];
    }
    int add(const T& o) { reserve(size()+1); v[size()-1]=o; return size()-1; }
    ~Block() { delete [] v; }
    void reset() { sizeD=mem=0; delete [] v; v=NULL; }
    T * getVector() { T * p=v; v=0; sizeD=mem=0; return p; }
};
// ----------------------------------------------------------------------------------
template <class T>
class Stack {
    Block<T> ref;
    int i;
  public:
    Stack() : i(0) { }
    bool isEmpty() { return !i; }
    void push(T obj) { ref.reserve(i+1); ref[i++]=obj; }
    T pop() { 
#     ifdef BlockCheck
        if(!i) throw EStack("pop() on empty"); 
#     endif
      return ref[--i]; 
    }
    T top() { 
#     ifdef BlockCheck
        if(!i) throw EStack("top() on empty"); 
#     endif
      return ref[i-1]; 
    }
};
// ----------------------------------------------------------------------------------
template <class T>
class FIFO {
    Block<T> ref;
    int ip,op;
  public:
    FIFO() : ip(0), op(0) { ref.reserve(1); }
    void put(T obj) { ref.reserve(op+1); ref[op++]=obj; }
    T get() { 
#     ifdef BlockCheck
        if(ip==op) throw EFIFO("get() on empty");
#     endif
      int i=ip++;
      if(op==ip) op=ip=0;
      return ref[i];
    }
    bool isEmpty() { return ip==op; }
};
// ----------------------------------------------------------------------------------
#define deletePtrs(block) \
  ({for(int i=0; i<block.size(); i++) if(block[i]) delete block[i];})
// ----------------------------------------------------------------------------------
#endif // BlockH



